/*****************************************************************************************
*	Single_wire.c Communicate with the TCA5405											 *
* 																						 *
* 	Author: Chris Kraft																	 *	
* 																						 *
* 	Contains functions to setup and communicate with the TCA5405 utilizing two different *
* 	methods. 																			 *
* 																						 *
* 																						 *
*  	H |-L -|-H -|-L -|-H -|-D4-|-D3-|-D2-|-D1-|-D0-| H 									 *
*	   -Initalize pulse      Data pulse MSB first--										 *
*																	                     *
*****************************************************************************************/

#include <msp430.h>
#include "Single_wire.h"
#include "BoosterPack.h"

//------------------------------------------------------------------
//	Global variables
//------------------------------------------------------------------
unsigned int  TCA5405_Data;				//Data to be sent over TimerA1 UART
unsigned char TCA5405_BitCnt;			//Number of bits left to send over UART


//------------------------------------------------------------------
//	This timer will be used to facilitate a UART like communication
//	with the TCA5405 will require P2.0 to be used
//------------------------------------------------------------------
void TCA5405_init(void)
{
    TA1CCTL0 = OUT;				//Timer A1 compare mode, outmode 0
	TA1CTL = TASSEL_2 + ID_3 + MC_2;						        //SMCLK continuous mode
}

//------------------------------------------------------------------
//	Main function to send a byte over TimerA1 UART
//------------------------------------------------------------------
void send_TCA5405_byte(unsigned char byte)
{
	while (TA1CCTL0 & CCIE);							// Wait for communication to finish
    TCA5405_BitCnt = 0xA;						        // 10 Bits to send
 
    TCA5405_Data = byte;                         		// Load global variable
    TCA5405_Data |= 0xA0;								// Append TCA5405 flag to data
    TCA5405_Data &= ~(0x100); 					// Add start bit
    
    TCA5405_Data <<= 1;                                 // Add space for stop bit
    TCA5405_Data |= 0x001;                           	// Add stop bit
    
    TA1CCR0 = TA1R;                           			// Current state of TA counter
    TA1CCR0 += TIME_NEXT_BIT;                   		// One bit time till first bit
    
    TA1CCTL0 = CCIS0 + OUTMOD0 + CCIE;                  // Set TXD on EQU0, Int


}

//--------------------------------------------------------------------
//	Timer A1 ISR for UART communication with TCA5405
//--------------------------------------------------------------------
void TCA5405_timer_ISR (void)
{
	TA1CCR0 += TIME_NEXT_BIT;                       // Add Offset to CCR0
    if (TA1CCTL0 & CCIS0)                           // TX on CCI0B?
    {
	    if ( TCA5405_BitCnt == 0)
	    {
	        TA1CCTL0 &= ~ CCIE;                     // All bits TXed, disable interrupt
	    }
	    else
	    {
	        TA1CCTL0 |=  OUTMOD2;                   // TX Space
	        if (TCA5405_Data & 0x200)
	        {
	            TA1CCTL0 &= ~ OUTMOD2;              // TX Mark
	        }
	        TCA5405_Data = TCA5405_Data << 1;
	        TCA5405_BitCnt --;
	    }
    }
}

//--------------------------------------------------------------------
//	Bit bang method to send a byte to the TCA5405 uses P1.4
//	not as effective as the timer UART method
//--------------------------------------------------------------------
void bit_bang_TCA5405_byte(char byte)
{				
	//Initialize single wire communication
	P2OUT &= ~DIN;					//Pull P1.4 LOW
	__delay_cycles(16);				//Wait 2uS
	P2OUT |= DIN;					//Set P1.4 HIGH
	__delay_cycles(16);				//Wait 2uS
	P2OUT &= ~DIN;					//Pull P1.4 LOW
	__delay_cycles(16);				//Wait 2uS
	P2OUT |= DIN;					//Set P1.4 HIGH
	__delay_cycles(10);				//Adjusted delay cycles for if statements				
	
	if(byte & 0x10)					//Is MSB 1
		P2OUT |= DIN;				//Set P1.4 HIGH (Transmit 1)
	else							
		P2OUT &= ~DIN;				//Set P1.4 LOW (Transmit 0)
	
	__delay_cycles(10);				
	
	if(byte & 0x08)		
		P2OUT |= DIN;
	else							
		P2OUT &= ~DIN;
				
	__delay_cycles(10);	
				
	if(byte & 0x04)		
		P2OUT |= DIN;
	else							
		P2OUT &= ~DIN;
						
	__delay_cycles(10);	
			
	if(byte & 0x02)		
		P2OUT |= DIN;
	else							
		P2OUT &= ~DIN;
				
	__delay_cycles(10);	
			
	if(byte & 0x01)		
		P2OUT |= DIN;
	else							
		P2OUT &= ~DIN;
	
	__delay_cycles(16);								
	
	P2OUT |= DIN;						//Reset P1.4 HIGH
}



