/********************************************************************************
*	main.c																		*
*																				*
* 	Main functions to control the TPL0401 Booster Pack							*	
*																				* 
* 	Author: 			Chris Kraft												*
*																				* 
* 	Revision Date: 		July 2011												*
*																				* 	
* 	Revision Level:		1st pass												*
* 																				*
*																				*
* 																				*
*   This code is to control the first run of the TPL0401 boards					*
* 	(the 12 with a blue wire change)											*
* 																				*
*   IMPORTANT NOTE: WDT interrupt can cause I2C communication to fail			*
* 	therefore I2C_Write and I2C_Read functions will disable the WDT interrupt,	*
* 	perform I2C function, and then enable WDT interrupt after completion		*
* 																				* 
********************************************************************************/
#include "msp430g2553.h"
#include "USCI_I2C.h"
#include "Timer_UART.h"
#include "BoosterPack.h"
#include "Single_wire.h"

//---------------------------------------------------------------
// Function prototypes contained in BoosterPack.h
//---------------------------------------------------------------
void init_I2C_TCA7408(void);										// Function of I2C writes to initalize TCA7408
void init_I2C_TLC59108(void);										// Function of I2C writes to initalize TLC59108

//---------------------------------------------------------------
// Global varible definitions
//---------------------------------------------------------------
unsigned char 	mode = LED_MODE;									// Mode flag

unsigned short 	update_TCA5405_delay=200;							// # of calls of the WDT before TCA5405 is flagged for update	
unsigned short 	update_TCA5405_counter=0;						    // Counter that keeps track of WDT calls			

unsigned short 	LEDon = BLUE;										// Gloabal variable that controls which LEDs are on									 
 
unsigned char 	POTAValue = 0, POTBValue = 0;						// Gloabal variables for GUI_MODE


// Gloabal tap values for LED color mixing 
unsigned char 	RedValues[25]	=	{60,60,30,15,5, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 5, 15, 30,60,60};	
unsigned char 	GreenValues[25] =	{21,21,21,21,21,21,21,21,21,21,21,21,60,60,10,5, 3, 1, 0, 0, 0, 0, 0, 0, 0 };
unsigned char 	BlueValues[25]  =	{0, 0, 0, 0, 0, 0, 0, 1, 5, 15, 30,60,21,21,21,21,21,21,21,21,21,21,21,21,21};
unsigned char 	index_RGB_LED = 0;									// Index value to be used with the above matricies 

unsigned char 	LED0_OUT = 0;										// Outputs for Register LED OUT 0 on the TLC59108
unsigned char   LED1_OUT = 0;										// Outputs for Register LED OUT 1 on the TLC59108
unsigned char   LED_POT_VALUE=0;									// Tap value to be sent to TPL0401B

unsigned char 	mplex_Counter = 0;									// Counter to facilitate LED color mixing

unsigned char   mcu_wake = CLEAR;									// Which interrupt cleared the LPM0 bits (WDT, BUTTON_PRESS, BYTE_IN_WAIT)




void main(void)
{
	
	unsigned char TCA5405_Val[20] = {0x00,0x01, 0x03, 0x07, 0x0F, 0x1F,0x1E,0x1C,0x18,0x10,0x00,0x10,0x18,0x1C,0x1E,0x1F, 0x0F, 0x07, 0x03,0x01};
	unsigned char 	TCAcount = 0;
	
	init_TPL0401_Booster();												// Initalize clock, GPIO, timers, interrupts of MSP430
    __delay_cycles(800);												// Wait for device power up before sending I2C commands
    __enable_interrupt();												// Enable interrupts for I2C and UART communication
	
	//I2C_Write(I2C_BYTE, TCA7408_ADDRESS, 0x01, 0x01);

	init_I2C_TCA7408();														// Initalize tca7408  (GPIO expander) 
	init_I2C_TLC59108();													// Initalize tlc59108 (LED Driver)
	
	I2C_Write(I2C_BYTE, TPL0401A_ADDRESS, TPL0401_REG, 0x00);					// Set TPL0401A tap value to zero
	I2C_Write(I2C_BYTE, TPL0401B_ADDRESS, TPL0401_REG, 0x00);					// Set TPL0401B tap value to zero
	
	I2C_Read(I2C_BYTE,TCA7408_ADDRESS, 0x01);										// Read configuartion register to clear interrupt reset bit
	I2C_Read(I2C_BYTE,TCA7408_ADDRESS, INTERRUPT_STATUS_REG);										// Read configuartion register to clear interrupt bit
	
	IE1 |= WDTIE;														// Enable WDT interrupt after init_I2C_TLC59108 to ensure correct function 
    IFG1 &= ~WDTIFG;													// (WDT interrupt cannot be called before init_I2C_TLC59108 risk damaging the LED)
	P1IFG &= ~BIT5;
	while (1)
	{
    	__bis_SR_register(LPM0_bits + GIE);								//Wait in low power mode for interrupts	
    	if(mode)
    	{
			 if (mcu_wake & WDT)
    		 {
    			mcu_wake &= ~WDT;														// Clear WDT wake bit
			 	
			 	I2C_Write(I2C_BYTE, TPL0401B_ADDRESS, TPL0401_REG, LED_POT_VALUE);		// Write new TPL0401B value
	    		I2C_Write(I2C_BYTE, TLC59108_ADDRESS, TLC59108_LED_OUT0, LED0_OUT);		// Turn on correct LEDs
	    		I2C_Write(I2C_BYTE, TLC59108_ADDRESS, TLC59108_LED_OUT1, LED1_OUT);
	    		
			 	if (update_TCA5405_counter >= update_TCA5405_delay)						
			 	{
					update_TCA5405_counter = 0;											// Reset TCA5405 delay counter
					I2C_Read(I2C_BYTE, TCA7408_ADDRESS, INTERRUPT_STATUS_REG);			// Clear interrupt flag from 7408 (ensures correct function of TCA7408 INT)
					if (TCAcount < 19)
						TCAcount++;
					else
						TCAcount = 0;
					//bit_bang_TCA5405_byte(TCA5405_Val[TCAcount]);						// Send value to TCA5405
					send_TCA5405_byte(TCA5405_Val[TCAcount]);
			 	}
    		}
    	}
    	else
    	{
			I2C_Read(I2C_BYTE, TCA7408_ADDRESS, INTERRUPT_STATUS_REG);					// Clear interrupt flag from TCA7408 register
    		if(mcu_wake & BYTE_IN_WAIT)														
	    	{
	    		TimerA_UART_tx(POTA_TX_FLAG);												// Send serial flag to GUI for A
	    		TimerA_UART_tx(POTAValue);											// Send TPL0401A tap value to GUI
	    		I2C_Write(I2C_BYTE, TPL0401A_ADDRESS, TPL0401_REG, POTAValue);	    // Write tap value to TPL0401A
	    		TimerA_UART_tx(POTB_TX_FLAG);												// Send serial flag to GUI for B
	    		TimerA_UART_tx(POTBValue);											// Send TPL0401B tap value to GUI
	    		I2C_Write(I2C_BYTE, TPL0401B_ADDRESS, TPL0401_REG, POTBValue);		// Write tap value to TPL0401B
	    		mcu_wake &= ~BYTE_IN_WAIT;											// Clear Byte in wait bit
	    	}
	    	if(mcu_wake & SEND_VALUES)												// TPL0401A\B values requested
	    	{
	    		POTAValue = I2C_Read(I2C_BYTE, TPL0401A_ADDRESS, TPL0401_REG);		// Read TPL0401A tap value
	    		POTBValue = I2C_Read(I2C_BYTE, TPL0401B_ADDRESS, TPL0401_REG);		// Read TPL0401B tap value
	    		TimerA_UART_tx(POTA_TX_FLAG);										// Send POTA serial flag
	    		TimerA_UART_tx(POTAValue);											// Send POTA tap value
	    		TimerA_UART_tx(POTB_TX_FLAG);										// Send POTB serial flag
	    		TimerA_UART_tx(POTBValue);											// Send POTB tap value
	    		mcu_wake &= ~SEND_VALUES;											// Clear send values bit
	    		
	    	}
    	}
    	if (mcu_wake & BUTTON_PRESS)
	    {
    		mcu_wake &= ~BUTTON_PRESS;												// Clear button press bit
    		P1IE &= ~BIT5;															// Disable bit3 interrupt (TCA7408 INT line)
    		decode_Button();														// Decode button press
	    	P1IE |= BIT5;															// Re-enable bit3 interrupt
	    }
	}
}

void init_I2C_TCA7408(void)
{	
	
	I2C_Write(I2C_BYTE, TCA7408_ADDRESS, TCA7408_OUTPUT_CONFIG, 0xF0);				// Set first four bits to inputs 4 MSBs to outputs
	I2C_Write(I2C_BYTE, TCA7408_ADDRESS, OUTPUT_REG, 0x00);							// Clear the output register
	I2C_Write(I2C_BYTE, TCA7408_ADDRESS, INPUT_DEFAULT_STATE, 0xFF);				// Set default state of inputs to high
	I2C_Write(I2C_BYTE, TCA7408_ADDRESS, RESISTOR_ENABLE, 0x00); 					// Disable pull/down up resistors 
}
void init_I2C_TLC59108(void)
{
	I2C_Write(I2C_BYTE, TLC59108_ADDRESS, 0x00, 0x00);								// 
	I2C_Write(I2C_BYTE, TLC59108_ADDRESS, 0x01, 0x80);								//
	I2C_Write(I2C_BYTE, TLC59108_ADDRESS, TLC59108_PWM0, 0x50);						// Set PWM for each output
	I2C_Write(I2C_BYTE, TLC59108_ADDRESS, TLC59108_PWM1, 0x1C);
	I2C_Write(I2C_BYTE, TLC59108_ADDRESS, TLC59108_PWM2, 0x47);
	I2C_Write(I2C_BYTE, TLC59108_ADDRESS, TLC59108_PWM5, 0x47);
	I2C_Write(I2C_BYTE, TLC59108_ADDRESS, TLC59108_PWM6, 0x1C);
	I2C_Write(I2C_BYTE, TLC59108_ADDRESS, TLC59108_PWM7, 0x50);
}

/********************************************************************************
*	void decode_Button(void)
* 
* 	Once a button press has been flagged by the Port 1 interrupt this function	*
* 	will be called to read the Input register of the TCA7408, determine what	*
* 	button was pressed and change the value of update_TCA5405_delay or 			*
* 	index_RGB_LED and LEDon														*
* 																				*
********************************************************************************/
void decode_Button(void)
{
	
	unsigned char TCA7408_inreg = I2C_Read(I2C_BYTE, TCA7408_ADDRESS, INPUT_STATUS_REG);
	I2C_Read(I2C_BYTE,TCA7408_ADDRESS, INTERRUPT_STATUS_REG);									// Clear TCA7408 interrupt register

	TCA7408_inreg = ~TCA7408_inreg;																
	if(TCA7408_inreg)
	{
		if(mode)
		{
			switch(TCA7408_inreg)
			{
				case 2:	if (update_TCA5405_delay > 25) 											// SW2 Pressed 		
							update_TCA5405_delay /= 2;											// Decrease delay between TCA5405 updates
						break;
				case 1: if(update_TCA5405_delay < 800)											// SW1 Pressed
							update_TCA5405_delay *= 2;											// Increase delay between TCA5405 updates
						break;
				case 8: switch(index_RGB_LED)													// SW4 Pressed
						{
							case 0: 	LEDon |= RED;											// Depending on index_RGB_LED
										break;													// Turn on and off specific LEDs
							case 11: 	LEDon &=~ BLUE;
										break;
							case 12:    LEDon |= GREEN;	
										break;
							case 23: 	LEDon &= ~RED;									
										break;
						}
						if (index_RGB_LED < 24)
							index_RGB_LED++;
						break;
				case 4: switch(index_RGB_LED)													// SW3 Pressed
						{
							case 24: 	LEDon |= RED;											// Depending on index_RGB_LED
										break;													// Turn on and off specific LEDs
							case 13: 	LEDon &=~ GREEN;
										break;
							case 12:    LEDon |= BLUE;	
										break;
							case 1: 	LEDon &= ~RED;											
										break;
						}
						if (index_RGB_LED > 0)
							index_RGB_LED--;
						break;
				case 6: mode = GUI_MODE;														// SW3 and SW2 Pressed enter GUI mode
						IFG1 &= ~WDTIFG;														// Clear WDT flag
						IE1 &= ~WDTIE;															// Disable WDT interrupt
						mcu_wake = CLEAR;														// Clear mcu_wake bits
						//bit_bang_TCA5405_byte(0x1F);											// Turn on D1-D5
						send_TCA5405_byte(0x1F);
    					I2C_Write(I2C_BYTE, TLC59108_ADDRESS, TLC59108_LED_OUT0, 0x00);			// Turn LEDs off
						I2C_Write(I2C_BYTE, TLC59108_ADDRESS, TLC59108_LED_OUT1, 0x00);	
						I2C_Write(I2C_BYTE, TPL0401B_ADDRESS, TPL0401_REG, 0x00);				// Reset REXT to zero
						I2C_Write(I2C_BYTE, TPL0401A_ADDRESS, TPL0401_REG, 0x00);				// Reset TPL0401A to zero
						break;
			}
		}
		else
		{
			if(TCA7408_inreg == 0x06)															// SW3 and SW2 Pressed return to LED mode
			{
				mode = LED_MODE;
				//bit_bang_TCA5405_byte(0x00);													// Turn off D1-D5
				send_TCA5405_byte(0x00);
				I2C_Write(I2C_BYTE, TPL0401B_ADDRESS, TPL0401_REG, 0x00);						// Reset REXT to zero
				IFG1 &= ~WDTIFG;																// Clear WDT flag
				IE1 |= WDTIE;																	// Enable WDT interrupt
				mcu_wake = CLEAR;																// Clear mcu_wake bits
			}
		}
	}
}


/********************************************************************************
*	Port 1 interrupt vector. Handles the INT line from the TCA5405				*
* 	that is attached to P1.3.													*
********************************************************************************/
#pragma vector=PORT1_VECTOR
__interrupt void PORT1_ISR_HOOK(void)
{
	P1IFG &= ~BIT5;												// Clear Interrupt flag on P1.3
	mcu_wake |= BUTTON_PRESS;									// Flag button press
	P1OUT ^= BIT0;
	_bic_SR_register_on_exit(LPM0_bits);						// Wake up on exit
}

/********************************************************************************
*	I2C Interrupt Vector														*
* 	I2C functions located in USCI_I2C.c											*
********************************************************************************/
#pragma vector=USCIAB0TX_VECTOR                          		
__interrupt void USCI0TX_ISR_HOOK(void)
{
	unsigned short lowPowerBits = 0;

    if (IFG2 & UCB0TXIFG) {
		/* USCI_B0 Transmit Interrupt Handler */
		lowPowerBits = USCIAB0TX_ISR();

		/* Exit low power mode based on return bits */
		__bic_SR_register_on_exit(lowPowerBits);
    }
    else {
		/* USCI_B0 Receive Interrupt Handler */
		USCIAB0RX_ISR();

		/* Enter active mode on exit */
		__bic_SR_register_on_exit(LPM4_bits);
    }
}

/********************************************************************************
*	TimerA0 CCR0 Interrupt vector												*
* 	Handles USB UART TX communication											*
* 	UART Functions are located in Timer_UART.c									*
********************************************************************************/
#pragma vector=TIMER0_A0_VECTOR
__interrupt void UART_TX_ISR(void)
{
	Timer_TX_ISR();
}
/********************************************************************************
*	TimerA0 CCR1 & CCR2 Interrupt vector										*
* 	CCR1 handles USB UART RX communication. CCR2 is unused						*
********************************************************************************/
#pragma vector=TIMER0_A1_VECTOR
__interrupt void UART_RX_ISR(void)
{
    static unsigned char rxBitCnt = 8;								// Recieve one byte
    static unsigned char rxData = 0;
	static unsigned char isNextByteA = 0, isNextByteB = 0;				
	
    switch (__even_in_range(TA0IV, TA0IV_TAIFG))   					// Use calculated branching
    { 
        case TA0IV_TACCR1:                         					// TACCR1 CCIFG - UART RX
            
            TA0CCR1 += Bitime;                     					// Add Offset to CCRx
            if (TA0CCTL1 & CAP)                   					// Capture mode = start bit edge
            {
                TA0CCTL1 &= ~CAP;                  					// Switch capture to compare mode
                TA0CCR1 += Bitime_5;       		   					// Point CCRx to middle of D0
            }
            else 
            {
                rxData >>= 1;
                if (TA0CCTL1 & SCCI)								// Get bit waiting in receive latch
                {            
                    rxData |= 0x80;
                }
                rxBitCnt--;
                if (rxBitCnt == 0) 									// All bits RXed?
                {    
                    if(isNextByteA)									// Was serial flag for TPL0401A recieved
                    {
                    	POTAValue = rxData;							// POTAValue equal to the RX data buffer
                    	isNextByteA= 0;								// Clear the A flag
                    	mcu_wake |= BYTE_IN_WAIT; 					// Flag byte in wait
                    }
                    else if (isNextByteB)							// Was serial flag for TPL0401B recieved
                    {
                    	POTBValue = rxData;							// POTBValue equal to the RX data buffer
                    	isNextByteB=0;								// Clear the B flag
                    	mcu_wake |= BYTE_IN_WAIT;					// Flag byte in wait
                    }
                    else 
                   	switch (rxData)									
                   	{
               			case POTA_RX_FLAG: 	isNextByteA=1;					// Serial flag for POTA recieved
			  					    isNextByteB=0;
           						    break;
               						    
               			case POTB_RX_FLAG:  isNextByteB=1;					// Serial flag for POTB recieved
									isNextByteA=0;
									break;
						case VALUE_REQUEST: 								// Request to send POTA and POTB values
									mcu_wake |= SEND_VALUES;
									break;
						case RESET_FLAG:									// Request to reset values
									isNextByteA = 0;
									isNextByteB = 0;
									POTAValue = 0;
									POTBValue = 0;
									mcu_wake |= BYTE_IN_WAIT;
									break;
               		
               		}
                    
                    rxBitCnt = 8;                  					// Re-load bit counter
                    TA0CCTL1 |= CAP;              					// Switch compare to capture mode
                    
                    __bic_SR_register_on_exit(LPM0_bits);  			// Wake up on exit
                }
            }
            break;
    }
}

/********************************************************************************
*	TimerA1 CCR0 Interrupt vector												*
* 	CCR0 handles TCA5405 communication through software timer.					*
* 	TCA5405 functions are located in Single_wire.c								*
********************************************************************************/
#pragma vector=TIMER1_A0_VECTOR
__interrupt void TCA5405_Timer_TX_ISR (void)
{
    TCA5405_timer_ISR();
}

/********************************************************************************
*	Watch dog timer Interrupt vector											*
* 	The following ISR is flagged every 1ms (1000Hz) to multiplex				*
* 	the ouptut of the LED driver. Every time the ISR is called					*
* 	update_TCA5405_counter is incrimented.										*
********************************************************************************/
#pragma vector=WDT_VECTOR
__interrupt void WDT_ISR_HOOK(void)
{
	mcu_wake |= WDT;															// Set WDT bit on mcu_wake
	if(~LEDon & GREEN)															
    {
	      mplex_Counter++;
	      if (mplex_Counter & 0x01)
	      {
	        
	        LED_POT_VALUE = RedValues[index_RGB_LED];
	       	LED0_OUT = (LEDon & RED) & 0xFF;
	       	LED1_OUT = (LEDon & RED) >> 8; 
	      }
	      else
	      {
	        
	        LED_POT_VALUE = BlueValues[index_RGB_LED];
	        LED0_OUT = (LEDon & BLUE) & 0xFF;
	       	LED1_OUT = (LEDon & BLUE) >> 8; 
	      }
     }
	 else if (LEDon & RED)
	 {
	  	mplex_Counter++;
	  	if (mplex_Counter & 0x01)
	  	{
	    	
	    	LED_POT_VALUE = RedValues[index_RGB_LED];
	    	LED0_OUT = (LEDon & RED) & 0xFF;
	       	LED1_OUT = (LEDon & RED) >> 8; 
	  	}
	  	else 
	  	{
	    	
	    	LED_POT_VALUE = GreenValues[index_RGB_LED];
	    	LED0_OUT = (LEDon & GREEN) & 0xFF;
	       	LED1_OUT = (LEDon & GREEN) >> 8; 
	  	} 
	 }
	 
	 else
	 {
	 	mplex_Counter++;
	 	if (mplex_Counter & 0x01)
	 	{
	    	
	    	LED_POT_VALUE = BlueValues[index_RGB_LED];
	    	LED0_OUT = (LEDon & BLUE) & 0xFF;
	       	LED1_OUT = (LEDon & BLUE) >> 8; 
	 	}
	  	else 
	  	{
	    	
	    	LED_POT_VALUE = GreenValues[index_RGB_LED];
	    	LED0_OUT = (LEDon & GREEN) & 0xFF;
	       	LED1_OUT = (LEDon & GREEN) >> 8; 
	  	} 	
	 }
     update_TCA5405_counter++;
 	__bic_SR_register_on_exit(LPM0_bits);
}

